/*******************************************************************************
 MRF24WG SPI Stub Functions

  Summary: Functions to control MRF24WG RESET and HIBERNATE pins need by the
           Universal Driver.  Functions in this module should not be called
           directly by the application code.

*******************************************************************************/

/* MRF24WG0M Universal Driver
*
* Copyright (c) 2012-2013, Microchip <www.microchip.com>
* Contact Microchip for the latest version.
*
* This program is free software; distributed under the terms of BSD
* license:
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* 1.    Redistributions of source code must retain the above copyright notice, this
*        list of conditions and the following disclaimer.
* 2.    Redistributions in binary form must reproduce the above copyright notice,
*        this list of conditions and the following disclaimer in the documentation
*        and/or other materials provided with the distribution.
* 3.    Neither the name(s) of the above-listed copyright holder(s) nor the names
*        of its contributors may be used to endorse or promote products derived
*        from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//==============================================================================
//                                  INCLUDES
//==============================================================================
// board-specific includes
#include <NetworkProfile.x>
#include "./ud_inc/shared/wf_universal_driver.h"
#include "../../DEIPcK/utility/System.h"
#include <Arduino.h>
#include <SPI.h>

/*****************************************************************************
  Function:
    void WF_SpiInit(void);

  Summary:
    Initializes SPI controller used to communicate with MRF24WG

  Description:
    Called by Universal Driver during initialization to initialize SPI interface.

 Parameters:
    None

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiInit(void)
{
   // volatile uint8_t rxData;
	SPI.begin();
	pinMode(WIF_SPI_SS, OUTPUT);
	digitalWrite(WIF_SPI_SS, HIGH);

//#if defined(__PIC32MZ__)
//    WF_INT_PPS();
//    WF_HIB_PPS();
//    WF_RESET_PPS();
//    WF_CS_PPS();
//    WF_SCK_PPS();
//    WF_SDI_PPS();
//    WF_SDO_PPS();
//#endif
//
    // The MRF24WG chip select line will be controlled using an I/O pin, not by
    // the SPI controller.
    //WF_CS_IO_PIN      = 1;     // set the level high (chip select disabled)
    //WF_CS_TRIS     = 0;     // Configure the I/O pin as an output and drive it
	/*PORTB = PORTB | 0x04;
	DDRB = DDRB | 0x04;
	PORTB = PORTB | 0x04;
*/
    // disable the three possible SPI interrupts (SP1 Fault, SP1 Rx done,
    // SPI1 Transfer Done).  Universal Driver does not require SPI interrupts.
//    SPI_IECxCLR = 0x03800000;  //
   // WIFI_SPI_INT_EIE     = 0;
   // WIFI_SPI_INT_TXIE    = 0;
   // WIFI_SPI_INT_RXIE    = 0;
	//SPCR = SPCR & 0x3F;	//disable SPI interrupt and the SPI controller
    // disable SPI controller
    //WIFI_SPICONbits.ON = 0;
	

    // clear the receive buffer
   /* rxData = SPDR;
    (void) rxData;  *///  prevent unsued warning
	
    // set the SPI baud rate to 10MHz
    //   Fsck = Fpb / (2 * (SPI1BRG + 1) = 80MHz / (2 * (3 + 1)) = 80MHz / 8 = 10MHz
    //WIFI_SPIBRG = max((((int) ((GetPeripheralClock() + WF_SPI_FREQ) / 2 / WF_SPI_FREQ)) - 1), 0);
                  
    // configure SPI1:
    //   * Master Mode enabled (MSTEN = 1)
    //   * Clock Polarity is idle high, active low (CKP = 1)
    //   * Serial output data changes on transition from idle clock state to active
    //       clock state, or high to low transition (CKE = 0)
    //   * Data sampled at end of output time (SMP = 1), on rising edge
    //WIFI_SPICON = 0x00000260;

	//SPCR SPI controller for Arduio Atmega328
/*	Bit 7	Bit 6	Bit 5	Bit 4	Bit 3	Bit 2	Bit 1	Bit 0
	SPIE	SPE		DORD	MSTR	CPOL	CPHA	SPR1	SPR0
	SPIE (SPI Interrupt Enable) bit: Set SPIE to one if you want the SPI interrupt to be executed when a serial transfer is completed.
	SPE (SPI Enable) bit: If you want to use the SPI, you must set this bit.
	DORD (Data Order) bit: You can choose in which order the data will be transmitted. Set DORD to one to send the least significant bit (LSB) first. 
		Set DORD to zero to send the most significant bit (MSB) first.
	MSTR (Master/Slave Select) bit: Set MSTR to configure the AVR as a Master SPI device. Clear MSTR to configure it as a Slave.
	CPOL (Clock Polarity) and CPHA (Clock Phase) bits: As stated previously, Master and Slave must agree on how to interpret the clock signal. 
		The first thing to do is to configure which logic level the clock will be in when the SPI is idle. 
		If CPOL is set to one, SCK is high when idle, and if CPOL is set to zero, SCK is low when idle. 
		The second thing is to configure during which clock transition the data will be sampled. 
		Set CPHA to sample the data on the trailing (last) edge, and clear CPHA to sample the data in the leading (first) edge.
		SPI Mode	CPOL	CPHA	Sample
		0			0		0		Leading (Rising) Edge
		1			0		1		Trailing (Falling) Edge
		2			1		0		Leading (Falling) Edge
		3			1		1		Trailing (Rising) Edge

		SPR1	SPR0	SCK frequency
		0		0		fosc/4
		0		1		fosc/16
		1		0		fosc/64
		1		1		fosc/128

	*/
	//SPCR = SPCR & B00111100;
	//SPCR = SPCR | B00011100;
 //   // enable SPI controller
 //   //WIFI_SPICONbits.ON = 1;
	//SPCR = SPCR | B01000000;
}

/*****************************************************************************
  Function:
    void WF_SpiEnableChipSelect(void);

  Summary:
    Selects the MRF24WG SPI by setting the CS line low.

  Description:
    Called by Universal Driver when preparing to transmit data to the MRF24WG.

 Parameters:
    None

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiEnableChipSelect(void)
{
    // if the SPI controller is being shared with another device then save its
    // context here and configure the SPI controller for the MRF24WG
    // (not needed for this hardware platform)

    // select the MRF24WG
    //WF_CS_IO = 0;
	//PORTB = PORTB & 0x0FB;
	digitalWrite(WIF_SPI_SS, LOW);
}

/*****************************************************************************
  Function:
    void WF_SpiDisableChipSelect(void);

  Summary:
    Deselects the MRF24WG SPI by setting CS high.

  Description:
    Called by Universal Driver after completing an SPI transaction.

 Parameters:
    None

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiDisableChipSelect(void)
{
    // deselect the MRF24WG
	//PORTB = PORTB | 0x04;
	digitalWrite(WIF_SPI_SS, HIGH);
    // if the SPI controller is being shared with another device then restore its
    // context here from the context that was saved in mrf_spi_enable_cs().
    // (not needed for this hardware platform)

}

/*****************************************************************************
  Function:
    void WF_SpiTxRx(uint8_t *p_txBuf, uint16_t txLength, uint8_t *p_rxBuf, uint16_t rxLength);

  Summary:
    Transmits and receives SPI bytes with the MRF24WG.

  Description:
    Called by Universal Driver to communicate with the MRF24WG.

 Parameters:
    p_txBuf  -- pointer to the transmit buffer
    txLength -- number of bytes to be transmitted from p_txBuf
    p_rxBuf  -- pointer to receive buffer
    rxLength -- number of bytes to read and copy into p_rxBuf

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiTxRx(const uint8_t *p_txBuf,
                uint16_t txLength,
                uint8_t *p_rxBuf,
                uint16_t rxLength)
{
    uint16_t byteCount;
    uint16_t i;
    volatile uint8_t  rxTrash;

	if (txLength > rxLength) {
		if (rxLength == 0) {
			for (int i = 0; i < txLength; i++)
			{
				SPI.transfer(p_txBuf[i]);
			}
		}
		else {
			int j = 0;
			for (; j < rxLength; j++)
			{
				p_rxBuf[j] = SPI.transfer(p_txBuf[j]);
			}
			for (; j < txLength; j++) {
				SPI.transfer(p_txBuf[j]);
			}
		}
	}
	else {
		if (txLength == 0) {
			for (int i = 0; i < rxLength; i++)
			{
				p_rxBuf[i]=SPI.transfer(0x0FF);
			}
		}
		else {
			int j = 0;
			for (; j < txLength; j++)
			{
				p_rxBuf[j] = SPI.transfer(p_txBuf[j]);
			}
			for (; j < rxLength; j++) {
				p_rxBuf[j] = SPI.transfer(0x0FF);
			}
		}
	}
}}
